import * as _backstage_backend_plugin_api from '@backstage/backend-plugin-api';
import { Config } from '@backstage/config';
import { TechInsightCheck, TechInsightsStore, TechInsightCheckRegistry, FactChecker, CheckValidationResponse } from '@backstage/plugin-tech-insights-node';
import { TopLevelCondition, Operator } from 'json-rules-engine';
import { Logger } from 'winston';
import { CheckResponse, BooleanCheckResult } from '@backstage/plugin-tech-insights-common';

/**
 * @public
 */
declare const JSON_RULE_ENGINE_CHECK_TYPE = "json-rules-engine";

/**
 * Sets a JsonRulesEngineFactCheckerFactory as FactCheckerFactory
 * loading checks from the config.
 *
 * @public
 */
declare const techInsightsModuleJsonRulesEngineFactCheckerFactory: () => _backstage_backend_plugin_api.BackendFeature;

/**
 * @public
 */
type Rule = {
    conditions: TopLevelCondition;
    name?: string;
    priority?: number;
};
/**
 * @public
 */
interface TechInsightJsonRuleCheck extends TechInsightCheck {
    rule: Rule;
}
/**
 * @public
 */
type CheckCondition = {
    operator: string;
    fact: string;
    factValue: any;
    factResult: any;
    result: boolean;
};
/**
 * @public
 */
type ResponseTopLevelCondition = {
    all: CheckCondition[];
} | {
    any: CheckCondition[];
};
/**
 * @public
 */
interface JsonRuleCheckResponse extends CheckResponse {
    rule: {
        conditions: ResponseTopLevelCondition & {
            priority: number;
        };
    };
}
/**
 * @public
 */
interface JsonRuleBooleanCheckResult extends BooleanCheckResult {
    check: JsonRuleCheckResponse;
}

/**
 * @public
 * Should actually be at-internal
 *
 * Constructor options for JsonRulesEngineFactChecker
 */
type JsonRulesEngineFactCheckerOptions = {
    checks: TechInsightJsonRuleCheck[];
    repository: TechInsightsStore;
    logger: Logger;
    checkRegistry?: TechInsightCheckRegistry<any>;
    operators?: Operator[];
};
/**
 * @public
 * Should actually be at-internal
 *
 * FactChecker implementation using json-rules-engine
 */
declare class JsonRulesEngineFactChecker implements FactChecker<TechInsightJsonRuleCheck, JsonRuleBooleanCheckResult> {
    private readonly checkRegistry;
    private repository;
    private readonly logger;
    private readonly validationSchema;
    private readonly operators;
    constructor(options: JsonRulesEngineFactCheckerOptions);
    runChecks(entity: string, checks?: string[]): Promise<JsonRuleBooleanCheckResult[]>;
    validate(check: TechInsightJsonRuleCheck): Promise<CheckValidationResponse>;
    getChecks(): Promise<TechInsightJsonRuleCheck[]>;
    private retrieveIndividualFactReferences;
    private ruleEngineResultsToCheckResponse;
    private static constructCheckResponse;
    private constructFactInformationResponse;
}
/**
 * @public
 *
 * Constructor options for JsonRulesEngineFactCheckerFactory
 *
 * Implementation of checkRegistry is optional.
 * If there is a need to use persistent storage for checks, it is recommended to inject a storage implementation here.
 * Otherwise, an in-memory option is instantiated and used.
 */
type JsonRulesEngineFactCheckerFactoryOptions = {
    checks: TechInsightJsonRuleCheck[];
    logger: Logger;
    checkRegistry?: TechInsightCheckRegistry<TechInsightJsonRuleCheck>;
    operators?: Operator[];
};
/**
 * @public
 *
 * Factory to construct JsonRulesEngineFactChecker
 * Can be constructed with optional implementation of CheckInsightCheckRegistry if needed.
 * Otherwise, defaults to using in-memory CheckRegistry.
 */
declare class JsonRulesEngineFactCheckerFactory {
    private readonly checks;
    private readonly logger;
    private readonly checkRegistry?;
    private readonly operators?;
    static fromConfig(config: Config, options: Omit<JsonRulesEngineFactCheckerFactoryOptions, 'checks'>): JsonRulesEngineFactCheckerFactory;
    constructor(options: JsonRulesEngineFactCheckerFactoryOptions);
    /**
     * @param repository - Implementation of TechInsightsStore. Used by the returned JsonRulesEngineFactChecker
     *                     to retrieve fact and fact schema data
     * @returns JsonRulesEngineFactChecker implementation
     */
    construct(repository: TechInsightsStore): JsonRulesEngineFactChecker;
}

export { CheckCondition, JSON_RULE_ENGINE_CHECK_TYPE, JsonRuleBooleanCheckResult, JsonRuleCheckResponse, JsonRulesEngineFactChecker, JsonRulesEngineFactCheckerFactory, JsonRulesEngineFactCheckerFactoryOptions, JsonRulesEngineFactCheckerOptions, ResponseTopLevelCondition, Rule, TechInsightJsonRuleCheck, techInsightsModuleJsonRulesEngineFactCheckerFactory as default };
